@@ -13,6 +13,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from threading import Event
+
 import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
@@ -20,6 +22,7 @@ import six
 
 from ceilometermiddleware import swift
 from ceilometermiddleware.tests import base as tests_base
+from keystoneauth1.fixture import keystoneauth_betamax as betamax
 
 
 class FakeApp(object):
@@ -90,6 +93,31 @@ class TestSwift(tests_base.TestCase):
             self.assertEqual('obj', metadata['object'])
             self.assertEqual('get', data[2]['target']['action'])
 
+    def test_get_background(self):
+        notified = Event()
+        app = swift.Swift(FakeApp(),
+                          {"nonblocking_notify": "True",
+                           "send_queue_size": "1"})
+        req = FakeRequest('/1.0/account/container/obj',
+                          environ={'REQUEST_METHOD': 'GET'})
+        with mock.patch('oslo_messaging.Notifier.info',
+                        side_effect=lambda *args, **kwargs: notified.set()
+                        ) as notify:
+            resp = app(req.environ, self.start_response)
+            self.assertEqual(["This string is 28 bytes long"], list(resp))
+            notified.wait()
+            self.assertEqual(1, len(notify.call_args_list))
+            data = notify.call_args_list[0][0]
+            self.assertEqual('objectstore.http.request', data[1])
+            self.assertEqual(28, data[2]['measurements'][0]['result'])
+            self.assertEqual('storage.objects.outgoing.bytes',
+                             data[2]['measurements'][0]['metric']['name'])
+            metadata = data[2]['target']['metadata']
+            self.assertEqual('1.0', metadata['version'])
+            self.assertEqual('container', metadata['container'])
+            self.assertEqual('obj', metadata['object'])
+            self.assertEqual('get', data[2]['target']['action'])
+
     def test_put(self):
         app = swift.Swift(FakeApp(body=['']), {})
         req = FakeRequest(
@@ -405,3 +433,31 @@ class TestSwift(tests_base.TestCase):
         with mock.patch('oslo_messaging.Notifier.info') as notify:
             list(app(req.environ, self.start_response))
             self.assertFalse(notify.called)
+
+    def test_ignore_projects_without_keystone(self):
+        app = swift.Swift(FakeApp(), {
+            'ignore_projects': 'cf0356aaac7c42bba5a744339a6169fa,'
+                               '18157dd635bb413c9e27686fee93c583',
+        })
+        self.assertEqual(["cf0356aaac7c42bba5a744339a6169fa",
+                          "18157dd635bb413c9e27686fee93c583"],
+                         app.ignore_projects)
+
+    @mock.patch.object(swift._LOG, 'warning')
+    def test_ignore_projects_with_keystone(self, warning):
+        self.useFixture(betamax.BetamaxFixture(
+            cassette_name='list_projects',
+            cassette_library_dir='ceilometermiddleware/tests/data',
+        ))
+        app = swift.Swift(FakeApp(), {
+            'auth_type': 'v2password',
+            'auth_url': 'https://[::1]:5000/v2.0',
+            'username': 'admin',
+            'tenant_name': 'admin',
+            'password': 'secret',
+            'ignore_projects': 'service,gnocchi',
+        })
+        self.assertEqual(["147cc0a9263c4964926f3ee7b6ba3685"],
+                         app.ignore_projects)
+        warning.assert_called_once_with(
+            "fail to find project '%s' in keystone", "gnocchi")
